@@ -1,4 +1,5 @@
 from getpass import getpass
+import errno
 import subprocess
 import sys
 from typing import Optional
@@ -19,23 +20,37 @@ def update_password_in_keychain(
         teamid: Optional[str] = None,
         keychain_password: Optional[str] = None,
 ):
-    label = label or service_name
+    security_add_generic_password_args = [
+        'security', 'add-generic-password',
+        '-a', account_name,  # Specify account name
+        '-s', service_name,  # Specify service name
+        '-w', password,  # Specify password to be added
+    ]
+    if label is not None:
+        security_add_generic_password_args += [
+            '-l', label,  # Specify label
+        ]
 
     result = subprocess.run(
-        [
-            'security', 'add-generic-password',
-            '-a', account_name,  # Specify account name
-            '-l', label,  # Specify label
-            '-s', service_name,  # Specify service name
-            '-w', password,  # Specify password to be added
-            # '-T', '/usr/bin/security',  # Specify an application which may access this item
-            # '-T', app_path,  # (multiple -T options are allowed)
-            '-U',  # Update item if it already exists
+        security_add_generic_password_args + [
+            '-T', app_path,  # Specify an application which may access this item
         ],
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
     )
-    if result.returncode != 0:
+    if result.returncode == errno.ENOTSUP:
+        # security: SecKeychainItemCreateFromContent (<default>): The specified item already exists in the keychain.
+        result = subprocess.run(
+            security_add_generic_password_args + [
+                '-U',  # Update item if it already exists
+            ],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )
+        if result.returncode != 0:
+            print(result.stderr.decode(), end='', file=sys.stderr, flush=True)
+            raise RuntimeError(result.returncode)
+    elif result.returncode != 0:
         print(result.stderr.decode(), end='', file=sys.stderr, flush=True)
         raise RuntimeError(result.returncode)
 
@@ -48,7 +63,7 @@ def update_password_in_keychain(
             'security', 'set-generic-password-partition-list',
             '-a', account_name,  # Specify account name
             '-s', service_name,  # Specify service name
-            '-S', f'apple-tool:,teamid:{teamid}',  # Comma-separated list of allowed partition IDs
+            '-S', f'teamid:{teamid}',  # Comma-separated list of allowed partition IDs
             '-k', keychain_password,  # The password for the keychain
         ],
         stdout=subprocess.PIPE,
